//*****************************************************************************
//  b_alert.cpp - functions to allow Small Eiffel to access the BeOS BAlert
//
//           This file is part of the SmallEiffel BeOS Library.
//                  Copyright (C) 1998 Lost Alternative
//                J. Scott Edwards - sedwards@xmission.com 
//
// This is  free  software;  you can  redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the
// Free Software Foundation;  either  version  2, or (at your option) any
// later verson.  It is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for  more  details.  You  should  have  received
// a copy of the GNU General Public  License  along  with this software;
// see the file COPYING.  If not, write to the  Free Software Foundation,
// Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// The original source code for SmallEiffel can be found at
//             http://www.loria.fr/SmallEiffel
//
//*****************************************************************************

#include <File.h>

#ifdef DEBUG
#include <stdio.h>
#endif


static uint32 modeTable[] = {
    B_READ_ONLY,
    B_WRITE_ONLY,
    B_READ_WRITE,
};


////////////////////////////////////////////////////////////////////////////////
// Function to construct a file
////////////////////////////////////////////////////////////////////////////////

extern "C" BFile* b_file_construct_path(
    const char* path,
    int         rwmode,          // 0=Read Only, 1=Write Only, 2=Read/Write
    bool        create,          // create file if it doesn't already exist
    bool        fail_if_exists,  // fail if file alread exists
    bool        erase,           // erase file if it exists
    bool        open_at_end);    // open with data pointer at the end of the file

BFile* b_file_construct_path(
    const char* path,
    int         rwmode,
    bool        create,
    bool        fail_if_exists,
    bool        erase,
    bool        open_at_end)
{
    uint32 openMode = modeTable[rwmode];
    if (create)         openMode |= B_CREATE_FILE;
    if (fail_if_exists) openMode |= B_FAIL_IF_EXISTS;
    if (erase)          openMode |= B_ERASE_FILE;
    if (open_at_end)    openMode |= B_OPEN_AT_END;

    BFile* theFile = new BFile(path, openMode);
#ifdef DEBUG
    printf("b_file_construct(%s,%d %d %d %d %d)->%08x\n",
           path, rwmode, create, fail_if_exists, erase, open_at_end, theFile);
#endif
    return theFile;
}


////////////////////////////////////////////////////////////////////////////////
// Function to destruct a file
////////////////////////////////////////////////////////////////////////////////

extern "C" void b_file_destruct(BFile* theFile);

void b_file_destruct(BFile* theFile)
{
#ifdef DEBUG
    printf("b_file_destruct(%08x)\n", theFile);
#endif
    delete theFile;
}


////////////////////////////////////////////////////////////////////////////////
// Function to call the InitCheck function of the file
////////////////////////////////////////////////////////////////////////////////

extern "C" int b_file_init_check(BFile* theFile);

int b_file_init_check(BFile* theFile)
{
    return ((int)theFile->InitCheck());
}


////////////////////////////////////////////////////////////////////////////////
// Function to get the size of the file
//
// WARNING! - files larger than 2GB cannot be handled because we must truncate
//            the size to an integer.
// Returns -1 if file is larger than 2GB (2,147,483,647 bytes)
////////////////////////////////////////////////////////////////////////////////

extern "C" int b_file_get_size(BFile* theFile);

int b_file_get_size(BFile* theFile)
{
    off_t size64;
    int32 size32;

    theFile->GetSize(&size64);

    if (size64 < 2147483648) {  // everything's cool
        size32 = size64;
    }
    else {                      // what the hell is stored in this file
        size32 = -1;
    }

#ifdef DEBUG
    printf("b_file_get_size(%08x)->%lld->%d\n", theFile, size64, size32);
#endif

    return size32;
}


////////////////////////////////////////////////////////////////////////////////
// Function to read a block of the file
////////////////////////////////////////////////////////////////////////////////

extern "C" int b_file_read(BFile* theFile, void* buffer, size_t size);

int b_file_read(BFile* theFile, void* buffer, size_t size)
{
#ifdef DEBUG
    printf("b_file_read(%08x, %08x, %d)\n", theFile, buffer, size);
#endif
    return ((int)theFile->Read(buffer, size));
}


