//*****************************************************************************
// b_window.cpp - functions to allow Small Eiffel to access the BeOS BWindow
//
//           This file is part of the SmallEiffel BeOS Library.
//                  Copyright (C) 1998 Lost Alternative
//                J. Scott Edwards - sedwards@xmission.com 
//
//  This is  free  software;  you can  redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by the
//  Free Software Foundation;  either  version  2, or (at your option) any
//  later verson.  It is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  General Public License for  more  details.  You  should  have  received
//  a copy of the GNU General Public  License  along  with this software;
//  see the file COPYING.  If not, write to the  Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//  The original source code for SmallEiffel can be found at
//             http://www.loria.fr/SmallEiffel
//
//*****************************************************************************

#include <Alert.h>
#include <Window.h>

#ifdef DEBUG
#include <stdio.h>
#endif

static window_type typeTable[] = {
    B_MODAL_WINDOW,
    B_BORDERED_WINDOW,
    B_TITLED_WINDOW,
    B_DOCUMENT_WINDOW
};


////////////////////////////////////////////////////////////////////////////////
// Define a Small Eiffel window class which saves the Small Eiffel target
// and the pointers to the callback routines.
////////////////////////////////////////////////////////////////////////////////

class SEWindow : public BWindow 
{
  public:
    SEWindow(void*       target,
             bool        (*quitRequestedCallback)(void*),
             void        (*messageReceivedCallback)(void*, uint32),
             BRect*      frame,
             const char* title,
             window_type type,
             uint32      flags
            );

    bool QuitRequested();
    void MessageReceived(BMessage* message);

  private:
    void* target;
    bool (*quitReqCallback)(void*);
    void (*msgRcvdCallback)(void*, uint32);

    bool ButtonMessage(BMessage* msg);   // routine to process a button message
};


////////////////////////////////////////////////////////////////////////////////
// C Function called by Small Eiffel to construct a window in the current
// workspace.
////////////////////////////////////////////////////////////////////////////////

extern "C" BWindow* b_window_construct(
    void*       target,
    bool        (*qrcb)(void*),
    void        (*mcb)(void*, uint32), // message callback
    BRect*      frame,
    const char* title,
    int         type,
    bool        movable,
    bool        resizable,
    bool        closable,
    bool        zoomable,
    bool        minimizable);

BWindow* b_window_construct(
    void*       target,
    bool        (*qrcb)(void*),
    void        (*mcb)(void*, uint32), // message callback
    BRect*      frame,
    const char* title,
    int         type,                   // 0=Modal, 1=Bordered, 2=Titled, 3=Document
    bool        movable,
    bool        resizable,
    bool        closable,
    bool        zoomable,
    bool        minimizable)
{
    uint32 flags = 0;
    if (!movable)     flags |= B_NOT_MOVABLE;
    if (!resizable)   flags |= B_NOT_RESIZABLE;
    if (!closable)    flags |= B_NOT_CLOSABLE;
    if (!zoomable)    flags |= B_NOT_ZOOMABLE;
    if (!minimizable) flags |= B_NOT_MINIMIZABLE;

    window_type wtype = typeTable[type];

    SEWindow* theWindow = new SEWindow(target,qrcb,mcb,frame,title,wtype,flags);
#ifdef DEBUG
    printf("b_window_construct(%f %f %f %f '%s' %d)->%08x\n",
        frame->left,frame->top,frame->right,frame->bottom, title, type, theWindow);
#endif
    return theWindow;
}


////////////////////////////////////////////////////////////////////////////////
// Small Eiffel window constructor
////////////////////////////////////////////////////////////////////////////////

SEWindow::SEWindow(void*       targ,
                   bool        (*qrcb)(void*),
                   void        (*mcb)(void*, uint32),
                   BRect*      frame,
                   const char* title,
                   window_type type,
                   uint32      flags) :
  BWindow(*frame, title, type, flags, B_CURRENT_WORKSPACE)
{
    target = targ;
    quitReqCallback = qrcb;
    msgRcvdCallback = mcb;
}


////////////////////////////////////////////////////////////////////////////////
// Function to call the window's show routine
////////////////////////////////////////////////////////////////////////////////

extern "C" void b_window_show(SEWindow* theWindow);

void b_window_show(SEWindow* theWindow)
{
#ifdef DEBUG
    printf("b_window_show(%08x)\n", theWindow);
#endif
    theWindow->Show();
}


////////////////////////////////////////////////////////////////////////////////
// Function to call the window's addChild
////////////////////////////////////////////////////////////////////////////////

extern "C" void b_window_add_child(SEWindow* theWindow, BView* aView);

void b_window_add_child(SEWindow* theWindow, BView* aView)
{
#ifdef DEBUG
    printf("b_window_add_child(%08x, %08x)\n", theWindow, aView);
#endif
    theWindow->AddChild(aView);
}


////////////////////////////////////////////////////////////////////////////////
// Small Eiffel Window-Quit Requested function that calls the Small Eiffel
// callback.
////////////////////////////////////////////////////////////////////////////////

bool SEWindow::QuitRequested()
{
    bool result = (*quitReqCallback)(target);
#ifdef DEBUG
    printf("SEWindow::QuitRequested: %d\n", result);
#endif
    return result;
}


////////////////////////////////////////////////////////////////////////////////
// Small Eiffel Window-MesssageReceived function that calls the Small Eiffel
// callback.
////////////////////////////////////////////////////////////////////////////////

void SEWindow::MessageReceived(BMessage* msg)
{
#ifdef DEBUG
    printf("SEWindow::MessageReceived(%08x)\n", msg->what);
#endif

    if (!ButtonMessage(msg)) {
        (*msgRcvdCallback)(target, msg->what);  // let eiffel handle it
    }
}


bool SEWindow::ButtonMessage(BMessage* msg)
{
    if (msg->what != 'bttn') return false;    // not a button message

    void* target;
    status_t target_status = msg->FindInt32("target", (int32*)&target);
    void  (*acb)(void*);
    status_t callback_status = msg->FindInt32("callback", (int32*)&acb);

    if (target_status != B_OK || callback_status != B_OK) { // something's amiss
        BAlert* alert = new BAlert(
              "bttn msg bad",
              "Button message is missing 'target' or 'callback' will be ignored",
              "OK",
              NULL,
              NULL,
              B_WIDTH_AS_USUAL,
              B_WARNING_ALERT
        );
        alert->Go();
        return true;   // even if we didn't like it,
                       //         it was still a button message so return true
    }

    (*acb)(target);    // pass it to the proper object in eiffel

    return true;
}

